@@ -41,6 +41,7 @@
 #include "strategyselector.h"
 #include "kvz_math.h"
 #include "fast_coeff_cost.h"
+#include "rate_control.h"
 
 static int encoder_control_init_gop_layer_weights(encoder_control_t * const);
 
@@ -153,6 +154,12 @@ encoder_control_t* kvz_encoder_control_init(const kvz_config *const cfg)
 
   // Take a copy of the config.
   memcpy(&encoder->cfg, cfg, sizeof(encoder->cfg));
+
+  // Copy the ROI file path
+  if (cfg->roi.file_path) {
+    encoder->cfg.roi.file_path = strdup(cfg->roi.file_path);
+  }
+
   // Set fields that are not copied to NULL.
   encoder->cfg.cqmfile = NULL;
   encoder->cfg.tiles_width_split = NULL;
@@ -596,6 +603,15 @@ encoder_control_t* kvz_encoder_control_init(const kvz_config *const cfg)
     memcpy(encoder->cfg.optional_key, cfg->optional_key, 16);
   }
 
+  if (encoder->cfg.target_bitrate > 0 && cfg->rc_algorithm != KVZ_NO_RC)
+  {
+    encoder->rc_data = kvz_get_rc_data(encoder);
+    if (!encoder->rc_data) {
+      fprintf(stderr, "Failed to initialize rate control.\n");
+      goto init_failed;
+    }
+  }
+
   return encoder;
 
 init_failed:
@@ -639,6 +655,8 @@ void kvz_encoder_control_free(encoder_control_t *const encoder)
     fclose(encoder->roi_file);
   }
 
+  kvz_free_rc_data(encoder->rc_data);
+
   free(encoder);
 }
 
