@@ -472,13 +472,13 @@ unsigned kvz_image_calc_satd(const kvz_picture *pic,
                              pic_data,
                              pic->stride,
                              ref_data,
-                             ref->stride) >> (KVZ_BIT_DEPTH - 8);
+                             ref->stride);
   } else {
     // Extrapolate pixels from outside the frame.
 
     // Space for extrapolated pixels and the part from the picture
     // The extrapolation function will set the pointers and stride.
-    kvz_pixel ext_buffer[LCU_LUMA_SIZE];
+    kvz_pixel ext_buffer[LCU_LUMA_SIZE + 1];
     kvz_pixel *ext = NULL;
     kvz_pixel *ext_origin = NULL;
     int ext_s = 0;
@@ -514,7 +514,7 @@ unsigned kvz_image_calc_satd(const kvz_picture *pic,
       pic_data,
       pic->stride,
       ext_origin,
-      ext_s) >> (KVZ_BIT_DEPTH - 8);
+      ext_s);
 
     return satd;
   }
