@@ -7,7 +7,7 @@
 set -eu
 . "${0%/*}/util.sh"
 
-common_args='-p4 --rd=0 --no-rdoq --no-signhide --subme=0 --deblock --sao --pu-depth-inter=1-3 --pu-depth-intra=2-3'
+common_args='--gop lp-g8d3t1 -p4 --rd=0 --no-rdoq --no-signhide --subme=0 --deblock --sao --pu-depth-inter=1-3 --pu-depth-intra=2-3'
 valgrind_test 264x130 10 $common_args -r1 --owf=1 --threads=0 --no-wpp
 valgrind_test 264x130 10 $common_args -r1 --owf=0 --threads=0 --no-wpp
 valgrind_test 264x130 10 $common_args -r2 --owf=1 --threads=2 --wpp
